@@ -16,11 +16,13 @@
 
 import argparse
 import itertools
+import textwrap
 from typing import Any
 
 import frozendict
 from googlecloudsdk.calliope import actions
 from googlecloudsdk.calliope import arg_parsers
+from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope.concepts import concepts
 from googlecloudsdk.calliope.concepts import deps
 from googlecloudsdk.command_lib.backupdr import util
@@ -333,14 +335,19 @@ def AddTriggerBackupFlags(parser):
   group.add_argument(
       '--custom-retention-days',
       required=False,
-      hidden=True,
       type=int,
       help=(
           'Duration for which backup data will be retained.'
-          'If not specified, the default retention period will be used.'
       ),
   )
 
+  base.Argument(
+      '--labels',
+      metavar='KEY=VALUE',
+      type=arg_parsers.ArgDict(value_type=str),
+      action=arg_parsers.UpdateAction,
+      help='Labels to be applied to the backup.').AddToParser(parser)
+
 
 def AddNetwork(parser, required=False):
   """Adds a positional network argument to parser.
@@ -550,7 +557,6 @@ def AddBackupRetentionInheritance(parser):
           'inherit-vault-retention',
           'match-backup-expire-time',
       ],
-      hidden=True,
       help=(
           'The inheritance mode for enforced retention end time of the backup'
           ' within this backup vault. Once set, the inheritance mode cannot be'
@@ -564,8 +570,8 @@ def AddBackupRetentionInheritance(parser):
 
 
 def AddUpdateBackupFlags(parser):
-  """Adds the --enforced-retention-end-time and --expire-time flags to the given parser."""
-  group = parser.add_argument_group('Update Backup Flags', required=True)
+  """Adds update backup flags to the given parser."""
+  group = parser.add_argument_group('Update Backup Flags')
   help_text = """
    Backups cannot be deleted until this time or later. This period can be extended, but not shortened.
    It should be specified in the format of "YYYY-MM-DD".
@@ -589,6 +595,44 @@ def AddUpdateBackupFlags(parser):
       ),
   )
 
+  # A backup can only be updated with one of these flags at a time.
+  label_update_group = parser.add_mutually_exclusive_group()
+
+  label_update_group.add_argument(
+      '--update-labels',
+      metavar='KEY=VALUE',
+      type=arg_parsers.ArgDict(value_type=str),
+      action=arg_parsers.UpdateAction,
+      help=textwrap.dedent("""\
+      A list of labels to apply to the backup.
+
+      Keys must start with a lowercase letter and contain only lowercase
+      letters, numbers, and hyphens, and must be between 1 and 63 characters
+      long. Values must contain only lowercase letters, numbers, and hyphens,
+      and must be between 0 and 63 characters long.
+
+      For example: `--update-labels=env=prod,team=storage`
+      """))
+
+  label_update_group.add_argument(
+      '--remove-labels',
+      metavar='KEY',
+      type=arg_parsers.ArgList(min_length=1),
+      help="""
+      A list of label keys to remove from the backup.
+
+      If a label does not exist, it is silently ignored.
+      """)
+
+  label_update_group.add_argument(
+      '--clear-labels',
+      action='store_true',
+      help="""
+      Remove all labels from the backup.
+
+      If the backup has no labels, this operation is a no-op.
+      """)
+
 
 def AddOutputFormat(parser, output_format):
   parser.display_info.AddFormat(output_format)
@@ -739,7 +783,6 @@ def AddKmsKey(parser):
   parser.add_argument(
       '--kms-key',
       required=False,
-      hidden=True,
       type=str,
       help=(
           'The Cloud KMS key resource name to be used for encryption. Format:'
@@ -781,7 +824,7 @@ def AddLogRetentionDays(parser, hidden=True):
   )
 
 
-def AddBackupRule(parser, required=True):
+def AddBackupRule(parser, required=False):
   """Adds a positional backup-rule argument to parser.
 
   Args:
@@ -1021,7 +1064,6 @@ def AddMaxCustomOnDemandRetentionDays(parser):
       '--max-custom-on-demand-retention-days',
       required=False,
       type=int,
-      hidden=True,
       help=("""Configure the maximum retention period for on-demand backups.
           The value must be greater than or equal to the minimum enforced
           retention period set on the backup vault."""),
